@@ -1,5 +1,5 @@
 /* 
- * Copyright 2016-2022 The Regents of the University of California
+ * Copyright 2016-2022, 2026 The Regents of the University of California
  * All rights reserved.
  * 
  * This file is part of Spoofer.
@@ -108,7 +108,7 @@ public:
       * Does not close the file descriptor.
       */
     ~PBStreamer() {
-	if (buf) delete buf;
+	if (buf) delete[] buf;
 #ifdef HAVE_PTHREAD
 	pthread_cond_destroy(&cond);
 	pthread_mutex_destroy(&mutex);
