@@ -1,5 +1,5 @@
 /* 
- * Copyright 2015-2022 The Regents of the University of California
+ * Copyright 2015-2022, 2026 The Regents of the University of California
  * All rights reserved.
  * 
  * This file is part of Spoofer.
@@ -19,9 +19,9 @@
  */
 
 /****************************************************************************
-   Program:     $Id: messages.cc,v 1.57 2022/02/07 19:48:20 kkeys Exp $
+   Program:     messages.cc
    Author:      Ken Keys, CAIDA
-   Date:        $Date: 2022/02/07 19:48:20 $
+   Date:        2026-01-12 15:34:52 -0800
    Description: Spoofer control channel message routines
 ****************************************************************************/
 #include "spoof.h"
