@@ -1,6 +1,6 @@
 /* 
  * Copyright 2004-2009 Rob Beverly
- * Copyright 2015-2022 The Regents of the University of California
+ * Copyright 2015-2022, 2026 The Regents of the University of California
  * All rights reserved.
  * 
  * This file is part of Spoofer.
@@ -20,10 +20,10 @@
  */
 
 /****************************************************************************
-   Program:     $Id: messages.h,v 1.17 2022/02/07 19:48:20 kkeys Exp $
+   Program:     messages.h
    Author:      Rob Beverly <rbeverly at csail.mit.edu>
                 Ken Keys, CAIDA
-   Date:        $Date: 2022/02/07 19:48:20 $
+   Date:        2026-01-12 15:34:52 -0800
    Description: Define control message types
 ****************************************************************************/
 
