@@ -1,6 +1,6 @@
 /* 
  * Copyright 2004-2009 Rob Beverly
- * Copyright 2015-2022 The Regents of the University of California
+ * Copyright 2015-2022, 2026 The Regents of the University of California
  * All rights reserved.
  * 
  * This file is part of Spoofer.
@@ -30,10 +30,10 @@
  */
 
 /****************************************************************************
-   Program:     $Id: net.cc,v 1.175 2022/02/07 19:48:20 kkeys Exp $
+   Program:     net.cc
    Author:      Rob Beverly <rbeverly at csail.mit.edu>
                 Ken Keys, CAIDA
-   Date:        $Date: 2022/02/07 19:48:20 $
+   Date:        2026-01-12 15:34:52 -0800
    Description: Spoofer utility routines
 ****************************************************************************/
 #include <stdarg.h>
@@ -166,14 +166,15 @@ public:
 //                     v0  v1  v2  v3  v4  v5  v6
 int spoof_layer[7] = { -1, -1, -1, -1,  0, -1,  0 };
 
-#define spoof_layer(sa) \
-    (*((sa)->sa_family == AF_INET6 ? &spoof_layer[6] : &spoof_layer[4]))
-    // note: spoof_layer(sa) is an lvalue
-
 static OptMutex spoof_layer_4_mutex;
 static OptMutex spoof_layer_6_mutex;
-#define spoof_layer_mutex(sa) \
-    ((sa)->sa_family == AF_INET6 ? spoof_layer_6_mutex : spoof_layer_4_mutex)
+#define spoof_layer_mutex(ipv) \
+    (ipv == 6 ? spoof_layer_6_mutex : spoof_layer_4_mutex)
+
+static inline bool spoof_layer_available(const int& ipv, const int& layer) {
+    LockGuard spoof_layer_guard(spoof_layer_mutex(ipv));
+    return spoof_layer[ipv] == 0 || spoof_layer[ipv] == layer;
+}
 
 // copy a sockaddr to a std::string (in binary form)
 struct saToStr {
@@ -313,11 +314,12 @@ const char *probe_info::dlt_str() const
 {
     if (!rtinfo || !rtinfo->ifinfo)
 	return "DLT undef"; // shouldn't happen
+    auto dlt = rtinfo->ifinfo->dlt;
 #ifdef HAVE_PCAP_DATALINK_VAL_TO_NAME
-    const char *result = pcap_datalink_val_to_name(rtinfo->ifinfo->dlt);
+    const char *result = pcap_datalink_val_to_name(dlt);
     if (result) return result;
 #endif
-    sprintf(dlt_buf, "DLT %d", rtinfo->ifinfo->dlt);
+    sprintf(dlt_buf, "DLT %d", dlt);
     return dlt_buf;
 }
 
@@ -510,6 +512,7 @@ static void get_route_info(struct probe_info *pinfo)
 	return;
     }
 
+    // Note: "memory leak" below is expected and ok
     pinfo->rtinfo = ref = new route_info(ifinfo);
 #endif
 }
@@ -578,19 +581,21 @@ bool init_sniffer(struct probe_info *pinfo, size_t pay_len)
 	printf("error in pcap_setnonblock: %s\n", errbuf);
 	goto error;
     }
-    pinfo->rtinfo->ifinfo->dlt = pcap_datalink(pinfo->sniffer);
-    debug(DEVELOP, "pcap_create %s ok, %s\n", ifname, pinfo->dlt_str());
 
-#ifdef DLT_LINUX_SLL
-    if (pinfo->rtinfo->ifinfo->dlt == DLT_LINUX_SLL) {
-	// Online userspace filters are known to be broken on LINUX_SLL (per
-	// https://github.com/the-tcpdump-group/libpcap/issues/184 and my own
-	// testing on a linux ppp iface), but they do _occasionally_ match, so
-	// we don't rely on a functional test and risk a false positive.
+    {
         LockGuard guard(pinfo->rtinfo->ifinfo->mutex);
-        pinfo->rtinfo->ifinfo->online_udp_filter_able = iface_info::BROKEN;
-    }
+        pinfo->rtinfo->ifinfo->dlt = pcap_datalink(pinfo->sniffer);
+        debug(DEVELOP, "pcap_create %s ok, %s\n", ifname, pinfo->dlt_str());
+#ifdef DLT_LINUX_SLL
+        if (pinfo->rtinfo->ifinfo->dlt == DLT_LINUX_SLL) {
+            // Online userspace filters are known to be broken on LINUX_SLL (per
+            // https://github.com/the-tcpdump-group/libpcap/issues/184 and my own
+            // testing on a linux ppp iface), but they do _occasionally_ match, so
+            // we don't rely on a functional test and risk a false positive.
+            pinfo->rtinfo->ifinfo->online_udp_filter_able = iface_info::BROKEN;
+        }
 #endif
+    }
 
     // Compile a capturing filter (preferably online)
     if (pinfo->spoof) {
@@ -701,8 +706,10 @@ struct probe_info *new_probe_info(int proto, bool spoof, socket_t nonSpoofSock,
 	return pinfo;
     }
 
-    if (!always_verify && spoof_layer(dst_sa) != 2 && spoof_layer(dst_sa) != 0)
-	return pinfo;
+    if (!always_verify) {
+        if (!spoof_layer_available(familytoipv(dst_sa->sa_family), 2))
+            return pinfo;
+    }
 
     if (!spoof) {
 	/* Identify the source address for routing to the destination. */
@@ -811,15 +818,16 @@ static void sniff_link(u_char *userdata,
     const u_char *pdu = get_ip_packet(pinfo, phdr, data);
     if (pdu) {
 	rtinfo->linklen = safe_int<size_t>(pdu - data);
+        // Note: "memory leak" below is expected and ok
 	rtinfo->linkhdr = new uint8_t[rtinfo->linklen];
 	memcpy(rtinfo->linkhdr, data, rtinfo->linklen);
+        LockGuard guard(rtinfo->ifinfo->mutex);
 	if (pinfo->proto == IPPROTO_UDP && pinfo->has_online_filter &&
 	    rtinfo->ifinfo->online_udp_filter_able == iface_info::UNKNOWN)
 	{
 	    // NB: TCP and UDP results may differ, and we care only about UDP.
 	    debug(DEVELOP, "Online filter works on %s (%s).\n",
 		pinfo->rtinfo->ifinfo->ifname, pinfo->dlt_str());
-	    LockGuard guard(rtinfo->ifinfo->mutex);
 	    rtinfo->ifinfo->online_udp_filter_able = iface_info::WORKS;
 	}
 	pinfo->layer_success = pinfo->layer_sent;
@@ -844,9 +852,10 @@ static void sniff_spoofed(u_char *userdata,
 	pinfo->setStatus(CONFIRMED);
 	int layer = pinfo->layer_success = pinfo->layer_sent;
 	// Update spoof_layer iff layer_success is more preferred
-	LockGuard spoof_layer_guard(spoof_layer_mutex(pinfo->dst_sa));
-	if (layer_pref(spoof_layer(pinfo->dst_sa)) > layer_pref(layer))
-	    spoof_layer(pinfo->dst_sa) = layer;
+        int ipv = familytoipv(pinfo->dst_sa->sa_family);
+	LockGuard spoof_layer_guard(spoof_layer_mutex(ipv));
+	if (layer_pref(spoof_layer[ipv]) > layer_pref(layer))
+	    spoof_layer[ipv] = layer;
     } else {
 	// Src was REWRITTEN
 	debug(HIGH, "(!M%d)", pinfo->layer_sent);
@@ -944,13 +953,15 @@ static int egress_pcap_dispatch(struct probe_info *pinfo, const char *label)
 }
 #endif // HAVE_PCAP
 
+static bool sl2p_flags[7] = { false, false, false, false, false, false, false };
+static bool sl2p_cached = false;
+static OptMutex sl2p_mutex;
+
 static bool spoof_layer2possible(const IPv &ipv)
 {
-    static bool flags[7] = { false, false, false, false, false, false, false };
 #ifdef HAVE_PCAP_SENDPACKET
-    static bool cached = false;
-
-    if (cached) return flags[ipv];
+    LockGuard guard(sl2p_mutex);
+    if (sl2p_cached) return sl2p_flags[ipv];
 
     pcap_if_t *alldevs = nullptr;
     char errbuf[PCAP_ERRBUF_SIZE];
@@ -978,14 +989,14 @@ static bool spoof_layer2possible(const IPv &ipv)
 	    case AF_INET:
 		if (memcmp(sa_ipaddr(daddr->addr), loopback4, IPV4ADDRLEN) == 0)
 		    continue; // ignore loopback address
-		flag = &flags[IPv4];
+		flag = &sl2p_flags[IPv4];
 		break;
 	    case AF_INET6:
 		if (memcmp(sa_ipaddr(daddr->addr), loopback6, IPV6ADDRLEN) == 0)
 		    continue; // ignore loopback address
 		if (nptohl(ua_field(sockaddr_in6, daddr->addr, sin6_scope_id)))
 		    continue; // ignore site-local and link-local addresses
-		flag = &flags[IPv6];
+		flag = &sl2p_flags[IPv6];
 		break;
 	    default:
 		continue; // ignore non-IP address
@@ -1007,13 +1018,13 @@ static bool spoof_layer2possible(const IPv &ipv)
 	    *flag = true;
 	}
     }
-    cached = true;
+    sl2p_cached = true;
 
 done:
     if (alldevs) pcap_freealldevs(alldevs);
 #else
 #endif // HAVE_PCAP_SENDPACKET
-    return flags[ipv];
+    return sl2p_flags[ipv];
 }
 
 enum ProbeStatus free_probe_info(struct probe_info *pinfo, bool skipSniffing)
@@ -1080,9 +1091,7 @@ bool initSockets(bool spoof, IPv ipv, socket_t& spoofsock, socket_t& udpsock,
     }
     if (!spoof)
 	return true;
-    if (ipv == IPv4 &&
-	(spoof_layer[ipv] == 0 || spoof_layer[ipv] == 3))
-    {
+    if (ipv == IPv4 && spoof_layer_available(ipv, 3)) {
 	// we want to at least try to spoof at layer 3 
 	spoofsock = newSocket(family, SOCK_RAW, IPPROTO_UDP);
 	if (spoofsock == INVALID_SOCKET) {
@@ -1094,7 +1103,7 @@ bool initSockets(bool spoof, IPv ipv, socket_t& spoofsock, socket_t& udpsock,
 		return true; // layer 3 spoofing is possible
 	}
     }
-    if (spoof_layer[ipv] == 0 || spoof_layer[ipv] == 2) {
+    if (spoof_layer_available(ipv, 2)) {
 	// we may want to spoof at layer 2 
 	if (spoof_layer2possible(ipv))
 	    return true; // layer 2 spoofing is possible
@@ -1315,120 +1324,6 @@ char *genSequence(char *buf, int len) {
     return buf;
 }
 
-
-#if 0
-/*!
- * \brief Subtract one struct timeval from another.
- *
- * Taken from:
- * http://www.gnu.org/software/hello/manual/libc/Elapsed-Time.html
- *
- * \param[out] result difference between the struct timevals at \a x and
- * \a y (x minus y).  This parameter may be NULL.
- *
- * \return 1 if the difference is negative, otherwise 0.
- */
-static int
-timeval_subtract(
-    struct timeval *resultPtr,
-    struct timeval x,
-    struct timeval y)
-{
-    struct timeval result;
-
-    /* Perform the carry for the later subtraction by updating y. */
-    if (x.tv_usec < y.tv_usec) {
-        int nsec = (y.tv_usec - x.tv_usec) / 1000000 + 1;
-        y.tv_usec -= 1000000 * nsec;
-        y.tv_sec += nsec;
-    }
-    if (x.tv_usec - y.tv_usec > 1000000) {
-        int nsec = (x.tv_usec - y.tv_usec) / 1000000;
-        y.tv_usec += 1000000 * nsec;
-        y.tv_sec -= nsec;
-    }
-
-    /* Compute the time remaining to wait.
-       tv_usec is certainly positive. */
-    result.tv_sec = x.tv_sec - y.tv_sec;
-    result.tv_usec = x.tv_usec - y.tv_usec;
-
-    if (resultPtr)
-        *resultPtr = result;
-
-    /* Return 1 if result is negative. */
-    return x.tv_sec < y.tv_sec;
-}
-
-/* 
-  Handy routines that follow taken directly from R. Stevens UNP 
-*/
-
-/*! \brief Read "n" bytes from a descriptor. */
-ssize_t readn(int fd, void *vptr, size_t n, time_t wait) {
-    size_t	nleft;
-    ssize_t	nread;
-    char	*ptr;
-    fd_set fds;
-    struct timeval deadline;
-
-    FD_ZERO(&fds);
-    FD_SET(fd, &fds);
-
-    if (gettimeofday(&deadline, NULL))
-        return -1;
-    deadline.tv_sec += wait;
-
-    ptr = vptr;
-    nleft = n;
-    while (nleft > 0) {
-        struct timeval timeout;
-        {
-            struct timeval now;
-            if (gettimeofday(&now, NULL))
-                return -1;
-            if (timeval_subtract(&timeout, deadline, now))
-                /* timed out */
-                break;
-        }
-        
-        int ret = 0;
-        if ((ret = select(fd + 1, &fds, NULL, NULL, &timeout)) < 0)
-            return -1;
-        else if (ret == 0)
-            /* timed out */
-            break;
-#ifdef _WIN32
-        if ( (nread = recv(fd, ptr, nleft, 0)) < 0)
-#else
-        if ( (nread = read(fd, ptr, nleft)) < 0)
-#endif
-	{
-            if (errno == EINTR)
-                nread = 0;		/* and call read() again */
-            else
-                return(-1);
-        } else if (nread == 0)
-            break;				/* EOF */
-
-        nleft -= nread;
-        ptr   += nread;
-    }
-    return(n - nleft);		/* return >= 0 */
-}
-/* end readn */
-
-ssize_t
-Readn(int fd, void *ptr, size_t nbytes, time_t wait)
-{
-	ssize_t		n;
-
-	if ( (n = readn(fd, ptr, nbytes, wait)) < 0)
-		warn("readn error: %s", SockLastErrmsg()());
-	return(n);
-}
-#else
-
 /* Rob's read function */
 size_t Readn(socket_t fd, void *vptr, size_t n, time_t timeout) {
     ssize_t nread = 0;
@@ -1462,7 +1357,6 @@ size_t Readn(socket_t fd, void *vptr, size_t n, time_t timeout) {
     }
     return(bytes_read);
 }
-#endif
 
 static ssize_t						/* Write "n" bytes to a descriptor. */
 writen(socket_t fd, const void *vptr, size_t n)
@@ -1607,14 +1501,16 @@ int socketSend(socket_t sock, const void *msg, size_t len,
     }
 
 #ifdef HAVE_PCAP
-    if (!pinfo->sniffer && (
-	// need verification
-	(always_verify && !pinfo->layer_success) ||
-	// need linkhdr
-	((spoof_layer(pinfo->dst_sa) == 0 || spoof_layer(pinfo->dst_sa) == 2) &&
-	    pinfo->rtinfo && !pinfo->rtinfo->linkhdr)))
-    {
-	init_sniffer(pinfo, len);
+    if (!pinfo->sniffer) {
+        int ipv = familytoipv(pinfo->dst_sa->sa_family);
+        if (// need verification
+            (always_verify && !pinfo->layer_success) ||
+            // need linkhdr
+            (pinfo->rtinfo && !pinfo->rtinfo->linkhdr &&
+                spoof_layer_available(ipv, 2)))
+        {
+            init_sniffer(pinfo, len);
+        }
     }
 #endif
 
@@ -1659,6 +1555,7 @@ int socketSendSpoof(socket_t spoofsock, socket_t udpsock, const void *msg,
 	severe("internal error: socketSendSpoof() with pinfo->spoof==false");
 	return -1;
     }
+    int ipv = familytoipv(dst->sa_family);
 
     // Note: If spoof_layer==0, the first thread to sniff a spoofed probe will
     // set spoof_layer, which may affect other threads.  (In older versions,
@@ -1668,7 +1565,7 @@ int socketSendSpoof(socket_t spoofsock, socket_t udpsock, const void *msg,
     // blocked; furthermore, one will likely succeed and set spoof_layer fast
     // enough that we'll know to not bother trying other APIs.
 
-    debug(DEVELOP, ">> socketSendSpoof: layer %d\n", spoof_layer(dst));
+    debug(DEVELOP, ">> socketSendSpoof: layer %d\n", spoof_layer[ipv]);
 
     if (pinfo->rtinfo && pinfo->rtinfo->linklen == route_info::FAILED) {
 	// Sniffing failed for a non-spoofed packet to same destination.
@@ -1694,7 +1591,7 @@ int socketSendSpoof(socket_t spoofsock, socket_t udpsock, const void *msg,
 
     for (int pref = 0; spoof_apis[pref].send; pref++) {
 	spoof_api_info &api = spoof_apis[pref];
-	if (!(spoof_layer(dst) == 0 || spoof_layer(dst) == api.layer))
+	if (!spoof_layer_available(ipv, api.layer))
 	    continue;
 
 	ssize_t sniff_len = -1;
@@ -1704,7 +1601,7 @@ int socketSendSpoof(socket_t spoofsock, socket_t udpsock, const void *msg,
 	    pay_len, ntopBuf(dst)(), ntohs(sa_port(dst)));
 
 #ifdef HAVE_PCAP
-	if (spoof_layer(dst) == 0) {
+	if (spoof_layer_available(ipv, 0)) {
 	    if (!api.attempt_printed) {
 		api.attempt_printed = true;
 		printf("Attempting to spoof IPv%d with %s (L%d)\n",
@@ -1724,7 +1621,7 @@ int socketSendSpoof(socket_t spoofsock, socket_t udpsock, const void *msg,
 #ifdef HAVE_PCAP
 	    if (!pinfo->sniffer) {
 		debug(DEVELOP, "not sniffing\n");
-	    } else if (spoof_layer(dst) != 0) {
+            } else if (!spoof_layer_available(ipv, 0)) {
 		// API is already known; defer sniffing til free_probe_info()
 		debug(DEVELOP, "defer sniffing\n");
 	    } else {
