@@ -1,5 +1,5 @@
 /* 
- * Copyright 2016-2022 The Regents of the University of California
+ * Copyright 2016-2022, 2026 The Regents of the University of California
  * All rights reserved.
  * 
  * This file is part of Spoofer.
@@ -19,8 +19,8 @@
  */
 
 /****************************************************************************
-   Program:     $Id: port.h,v 1.20 2022/02/07 19:48:20 kkeys Exp $
-   Date:        $Date: 2022/02/07 19:48:20 $
+   Program:     port.h
+   Date:        2026-01-12 15:34:52 -0800
    Description: general portability.  This file should be #included after
                 system headers and headers of other packages we depend on,
                 but before spoofer headers.
