@@ -1,6 +1,6 @@
 /* 
  * Copyright 2004-2009 Rob Beverly
- * Copyright 2015-2022 The Regents of the University of California
+ * Copyright 2015-2022, 2026 The Regents of the University of California
  * All rights reserved.
  * 
  * This file is part of Spoofer.
@@ -30,8 +30,8 @@
  */
 
 /****************************************************************************
-   Program:     $Id: spoof.h,v 1.145 2022/02/07 19:48:20 kkeys Exp $
-   Date:        $Date: 2022/02/07 19:48:20 $
+   Program:     spoof.h
+   Date:        2026-01-12 15:34:52 -0800
    Description: spoofer header
                 Should be included after system and third-party headers,
                 but before any other spoofer headers.
@@ -364,7 +364,7 @@ static inline unsigned int addrlen(const family_t &family)
     return
 	(family == AF_INET6) ? IPV6ADDRLEN :
 	(family == AF_INET) ? IPV4ADDRLEN :
-	0;
+	IPV6ADDRLEN; // shouldn't happen, but if it does, return the max value
 }
 
 static inline IPv familytoipv(const family_t &family)
@@ -570,16 +570,6 @@ NORETURN void fatal(const char *, ...) FORMAT_PRINTF(1, 2);
 NORETURN void pfatal(const char *, ...) FORMAT_PRINTF(1, 2);
 void streamSetup(void);
 char *timestring(void); // Uses static buffer - not threadsafe!
-uint32_t n6ton4(const unsigned char *addr) ATR_PURE ATR_NONNULL();
-uint32_t n6toh4(const unsigned char *addr) ATR_PURE ATR_NONNULL();
-int n4ton6(const uint32_t ip, unsigned char *addr) ATR_PURE ATR_NONNULL();
-int h4ton6(const uint32_t ip, unsigned char *addr) ATR_PURE ATR_NONNULL();
-int n6tohexstring(const unsigned char *addr, char *res) ATR_NONNULL();
-int isIPv4(const unsigned char *addr) ATR_PURE ATR_NONNULL();
-int isIPv6(const unsigned char *addr) ATR_PURE ATR_NONNULL();
-char *printIPv6(const unsigned char *a) ATR_NONNULL();
-int isIPv6Equal(const unsigned char *a1, const unsigned char *a2) ATR_PURE ATR_NONNULL();
-const char *strstrPortable(const char *a, const char *b, size_t n) ATR_PURE ATR_NONNULL();
 
 // example:   printf("time: %s\n", gmTimeStr()());
 class gmTimeStr {
