@@ -1,6 +1,6 @@
 /* 
  * Copyright 2004-2009 Rob Beverly
- * Copyright 2015-2022 The Regents of the University of California
+ * Copyright 2015-2022, 2026 The Regents of the University of California
  * All rights reserved.
  * 
  * This file is part of Spoofer.
@@ -20,10 +20,10 @@
  */
 
 /****************************************************************************
-   Program:     $Id: util.cc,v 1.65 2022/02/07 19:48:21 kkeys Exp $
+   Program:     util.cc
    Author:      Rob Beverly <rbeverly at csail.mit.edu>
                 Ken Keys, CAIDA
-   Date:        $Date: 2022/02/07 19:48:21 $
+   Date:        2026-01-12 15:34:52 -0800
    Description: Spoofer utility routines
 ****************************************************************************/
 
@@ -300,90 +300,3 @@ void streamSetup(void) {
         perror("dup2");
     }
 }
-
-/* Unified IPv4/IPv6 address field helper routines.
- *   Ex:
- *     char bob[IPV6ADDRLEN];
- *     uint32_t alice = 91232354;
- *     h4to6n(alice, bob);
- *     printf("Alice: %u %u\n", alice, n6toh4(bob));
- */
-/* network byte-order IPv6 to network byte-order IPv4 */
-uint32_t n6ton4(const unsigned char *addr) {
-    const unsigned char *begin = addr + (IPV6ADDRLEN - IPV4ADDRLEN);
-    uint32_t res;
-    memcpy(&res, begin, IPV4ADDRLEN);
-    return (res);
-}
-
-/* network byte-order IPv6 to host byte-order IPv4 */
-uint32_t n6toh4(const unsigned char *addr) {
-    return (ntohl(n6ton4(addr)));
-}
-
-/* network byte-order IPv4 to network byte-order IPv6 */
-int n4ton6(const uint32_t ip, unsigned char *addr) {
-    unsigned char *begin = addr + (IPV6ADDRLEN - IPV4ADDRLEN);
-    memcpy(begin, &ip, IPV4ADDRLEN);
-    return TRUE;
-}
-
-/* host byte-order IPv4 to network byte-order IPv6 */
-int h4ton6(const uint32_t ip, unsigned char *addr) {
-    unsigned char *begin = addr + (IPV6ADDRLEN - IPV4ADDRLEN);
-    uint32_t res = htonl(ip);
-    memcpy(begin, &res, IPV4ADDRLEN);
-    return TRUE;
-}
-
-int n6tohexstring(const unsigned char *addr, char *res) {
-    int i=0;
-    snprintf(res, 3, "0x");
-    for (i=1;i<=IPV6ADDRLEN;i++) {
-        snprintf(res + (i*2), 3, "%02x", *addr);
-        addr++;
-    }
-    return TRUE;
-}
-
-int isIPv4(const unsigned char *addr) {
-    int i=0;
-    const unsigned char *d = addr;
-    for (i=0;i<(IPV6ADDRLEN - IPV4ADDRLEN);i++) {
-        if (*d++ != 0x00)
-            return FALSE;
-    }
-    return TRUE;
-}
-
-int isIPv6(const unsigned char *addr) {
-    return (isIPv4(addr) ? FALSE : TRUE);
-}
-
-int isIPv6Equal(const unsigned char *a1, const unsigned char *a2) {
-    int ret = memcmp(a1, a2, IPV6ADDRLEN);
-    return (ret == 0 ? TRUE : FALSE);
-}
-
-char *printIPv6(const unsigned char *a) {
-    static char v6string[SMALLBUF];
-    inet_ntop(AF_INET6, a, v6string, SMALLBUF);
-    return v6string;
-}
-
-// portable version of BSD's strnstr()
-const char *strstrPortable(const char *a, const char *b, size_t n) {
-    size_t alen, blen, i;
-    // don't depend on there being a \0 within the first n chars
-    for (alen = 0; alen < n; alen++)
-	if (a[alen] == '\0') break;
-    blen = strlen(b);
-    if (blen > alen) return nullptr;
-    if (n > alen - blen)
-	n = alen - blen;
-    for (i = 0; i <= n; i++) {
-	if (memcmp(a+i, b, blen) == 0)
-	    return a+i;
-    }
-    return nullptr;
-}
