@@ -1,5 +1,5 @@
 /* 
- * Copyright 2015-2022 The Regents of the University of California
+ * Copyright 2015-2022, 2026 The Regents of the University of California
  * All rights reserved.
  * 
  * This file is part of Spoofer.
@@ -38,7 +38,6 @@
 #endif
 #include "../../config.h"
 #include "common.h"
-static const char cvsid[] ATR_USED = "$Id: common.cpp,v 1.94 2022/03/04 23:48:09 kkeys Exp $";
 
 SpooferBase::OnDemandDevice SpooferBase::outdev(stdout);
 SpooferBase::OnDemandDevice SpooferBase::errdev(stderr);
@@ -100,7 +99,7 @@ bool getProcessName(qint64 pid, char *buf, unsigned len)
 	} else {
 	    buf[0] = '\0';
 	}
-	fclose(ps);
+	pclose(ps);
 	return true;
     }
 
@@ -190,6 +189,8 @@ SpooferBase::Config::Config() :
 	QSL(DESC_SHARE_REMEDY)),
     enableTLS(QSL("enableTLS"), true,
 	QSL("Use SSL/TLS to connect to server (recommended unless blocked by your provider)")),
+    singleThreaded(QSL("singleThreaded"), false,
+	QSL("Force prober to run single threaded (slower, but uses less memory)")),
     // Probing
     netPollInterval(QSL("netPollInterval"), 2*60, 1, 86400,
 	QSL("Wait to check for a network change (seconds)")),
