@@ -1,5 +1,5 @@
 /* 
- * Copyright 2015-2022 The Regents of the University of California
+ * Copyright 2015-2022, 2026 The Regents of the University of California
  * All rights reserved.
  * 
  * This file is part of Spoofer.
@@ -47,7 +47,6 @@
 #include "app.h"
 #include "BlockWriter.h"
 
-static const char cvsid[] ATR_USED = "$Id: app.cpp,v 1.226 2022/03/04 23:48:09 kkeys Exp $";
 
 const QString App::appname(QSL(APPNAME));
 const QString App::schedulerLogFtime = QSL("'" APPNAME "-'yyyy~MM~dd-HH~mm~ss'.txt'");
@@ -696,6 +695,7 @@ bool App::init(int &exitCode)
 	QDir qdir(dataDir);
 	removeFiles(qdir, proberLogGlob);
 	removeFiles(qdir, schedulerLogGlob);
+	QFile::remove(dataDir % QSL("/upgrade-log.txt"));
 	QString dataDirName = qdir.dirName();
 	qdir.cdUp();
 	if (qdir.rmdir(dataDirName)) {
@@ -710,8 +710,9 @@ bool App::init(int &exitCode)
 
     // Delete settings (after getting settings lock)
     if (optDeleteSettings) {
+	QString filename = config->fileName(); // copy before it disappears
 	config->remove();
-	sperr << "Deleted settings in " << config->fileName() << Qt_endl;
+	sperr << "Deleted settings in " << filename << Qt_endl;
 	exitCode = SP_EXIT_OK; // success
 	return false; // caller should not do app.exec()
     }
@@ -1094,7 +1095,7 @@ void App::uiRead()
 	} else if (strcmp(data, "upgrade") == 0) {
 	    if (!opAllowedVerbose(ui, config->unprivPref)) continue;
 	    if (upgradeInfo) {
-		startUpgrade(ui);
+		startUpgrade();
 	    } else {
 		qDebug() << "No upgrade available.";
 		const sc_msg_text msg(QSL("No upgrade available."));
@@ -1147,6 +1148,7 @@ void App::startProber(bool manual)
     if (config->enableIPv4()) args << QSL("-4");
     if (config->enableIPv6()) args << QSL("-6");
     if (!config->enableTLS()) args << QSL("--no-tls");
+    if (config->singleThreaded()) args << QSL("-1");
 #ifdef AUTOUPGRADE_ENABLED
     upgradeState = US_NONE;
     if (config->autoUpgrade() && config->enableTLS())
@@ -1265,12 +1267,13 @@ void App::proberFinished(int exitCode, QProcess::ExitStatus exitStatus)
 		exitCode << (int)exitStatus;
     }
 
-    // UIs expect SC_UPGRADE_AVAILABLE after SC_PROBER_FINISHED
-    parseProberTextForUpgrade();
-
     recordRun(success); // before deleting prober
     deleteProber();
 
+    // Prober must be done before we can start upgrade.
+    // UIs expect SC_UPGRADE_AVAILABLE after SC_PROBER_FINISHED
+    parseProberTextForUpgrade();
+
     QDir dir(dataDir);
     int nlogs = config->keepLogs();
     if (nlogs > 0) {
@@ -1481,25 +1484,35 @@ void App::cancelUpgrade()
     }
 }
 
-void App::startUpgrade(QLocalSocket *requestor)
+void App::startUpgrade()
 {
-    if (prober) {
-	if (requestor)
-	    BlockWriter(requestor) << (qint32)SC_ERROR <<
-		sc_msg_text(QSL("Can't upgrade while prober is running."));
-	qWarning() << "Can't upgrade while prober is running.";
-	return;
-    }
     if (!upgradeInfo) {
+	for (QLocalSocket *ui : uiSet) {
+	    sendUpgradeError(ui, QSL("Internal error: startUpgrade() without upgradeInfo"));
+	}
 	qWarning() << "Internal error: startUpgrade() without upgradeInfo";
 	return;
     }
+
+    // Never automatically retry this failed upgrade version (in this
+    // scheduler's lifetime), so we don't get stuck in a loop of failed
+    // upgrades and failed probers.
+    if (upgradeIfGreater < upgradeInfo->vnum)
+        upgradeIfGreater = upgradeInfo->vnum;
+
     sperr << "Upgrading to " << upgradeInfo->vstr << Qt_endl;
     upgradeInfo->autoTime = -1;
     upgradePromptTimer.stop();
     QFile::remove(upFinName());
     upFinRemover.reset();
     QFile::remove(dataDir % QSL("/upgrade-log.txt"));
+    if (prober) {
+	for (QLocalSocket *ui : uiSet) {
+	    sendUpgradeError(ui, QSL("Can't upgrade while prober is running."));
+	}
+	qWarning() << "Can't upgrade while prober is running.";
+	return;
+    }
 
 #ifdef UPGRADE_WITHOUT_DOWNLOAD
     // execute an installer that does not need to be downloaded first
@@ -1557,11 +1570,6 @@ void App::abortInstallation(const QString &err)
 	sendUpgradeError(ui, err);
 	qDebug() << "sent SC_UPGRADE_ERROR to ui";
     }
-    // Never automatically retry this failed installer version (in this
-    // scheduler's lifetime), so we don't get stuck in a loop of failed
-    // installers and failed probers.
-    if (upgradeIfGreater < upgradeInfo->vnum)
-	upgradeIfGreater = upgradeInfo->vnum;
     if (upgradeState >= US_INSTALLING) {
 	// UIs have been stopped, so we must write something to trigger them
 	// to restart in case the upgrade process hasn't written anything yet.
