@@ -1,5 +1,5 @@
 /* 
- * Copyright 2015-2022 The Regents of the University of California
+ * Copyright 2015-2022, 2026 The Regents of the University of California
  * All rights reserved.
  * 
  * This file is part of Spoofer.
@@ -209,7 +209,7 @@ protected:
 #endif
 #ifdef AUTOUPGRADE_ENABLED
     void cancelUpgrade();
-    void startUpgrade(QLocalSocket *ui = nullptr);
+    void startUpgrade();
 #endif
 
 private slots:
