@@ -1,5 +1,5 @@
 /* 
- * Copyright 2016-2022 The Regents of the University of California
+ * Copyright 2016-2022, 2026 The Regents of the University of California
  * All rights reserved.
  * 
  * This file is part of Spoofer.
@@ -20,7 +20,7 @@
 
 /****************************************************************************
    Author:      Ken Keys, CAIDA
-   Date:        $Date: 2022/02/07 19:48:23 $
+   Date:        2026-01-12 15:42:06 -0800
    Description: get application name
 ****************************************************************************/
 
@@ -37,7 +37,6 @@
 #include <libgen.h>
 #include "AppInfo.h"
 
-static const char cvsid[] ATR_USED = "$Id: AppInfo.cc,v 1.14 2022/02/07 19:48:23 kkeys Exp $";
 
 char *AppInfo::appPath = nullptr;
 char *AppInfo::appDir = nullptr;
@@ -46,13 +45,14 @@ char *AppInfo::appDir = nullptr;
 AppInfo::AppInfo(const char *argv0)
 {
     if (appPath) return;
-    char buf[PATH_MAX];
+    char buf[PATH_MAX+1] = "";
+    ssize_t n;
 
     // Get the app name from /proc on systems that have it.
-    if ((readlink("/proc/self/exe", buf, PATH_MAX) > 0) ||        // Linux
-        (readlink("/proc/curproc/file", buf, PATH_MAX) > 0) ||    // FreeBSD
-        (readlink("/proc/self/path/a.out", buf, PATH_MAX) > 0))   // Solaris
-	    { appPath = strdup(buf); return; }
+    if (((n = readlink("/proc/self/exe", buf, PATH_MAX)) > 0) ||      // Linux
+        ((n = readlink("/proc/curproc/file", buf, PATH_MAX)) > 0) ||  // FreeBSD
+        ((n = readlink("/proc/self/path/a.out", buf, PATH_MAX)) > 0)) // Solaris
+	    { buf[n] = '\0'; appPath = strdup(buf); return; }
 
     // Absolute argv[0]
     if (argv0[0] == '/') {
