@@ -1,5 +1,5 @@
 /* 
- * Copyright 2016-2022 The Regents of the University of California
+ * Copyright 2016-2022, 2026 The Regents of the University of California
  * All rights reserved.
  * 
  * This file is part of Spoofer.
@@ -20,7 +20,7 @@
 
 /****************************************************************************
    Author:      Ken Keys, CAIDA
-   Date:        $Date: 2022/02/07 19:48:23 $
+   Date:        2026-01-12 15:42:06 -0800
    Description: Traceroute wrapper
 ****************************************************************************/
 
@@ -29,7 +29,6 @@
 #include "AppInfo.h"
 #include "routetracer.h"
 
-static const char cvsid[] ATR_USED = "$Id: routetracer.cc,v 1.21 2022/02/07 19:48:23 kkeys Exp $";
 
 // per-IPv RouteTracer configuration
 bool RouteTracer::initializing[7] =
@@ -186,22 +185,27 @@ void RouteTracer::Child::parse(Tracker *tracker)
 	if (scanned < result->length && result->text[scanned] == '\n') {
 	    // normal end of line
 	} else if (this->done && scanned == result->length) {
-	    scanned++; // end of unterminated final line
-	} else
+            if (parsed == result->length)
+                break; // nothing left
+	    // end of unterminated final line
+	} else {
 	    continue; // middle of a line
+        }
 
 	const char *line = result->text + parsed;
-
 	size_t len = scanned - parsed;
 
-	if (type[rt->ipv] == SCAMPER && strncmp(line, "traceroute from", sizeof("traceroute from") - 1) == 0) {
+	if (type[rt->ipv] == SCAMPER && len > sizeof("traceroute from") &&
+            strncmp(line, "traceroute from", sizeof("traceroute from") - 1) == 0)
+        {
 	    // Beginning of new scamper target output
 	    adjustScamperGoal(tracker); // update goal of previous target
 	    latest_hop = 0;
 	    current_goal = TRACEROUTE_ITEM_COST;
 	} else {
 	    char *t;
-	    long hop = strtol(line, &t, 10);
+            while (len > 0 && isspace(*line)) { line++; len--; } // skip space
+	    long hop = len > 0 ? strtol(line, &t, 10) : 0L;
 	    if (hop >= 1 && hop <= TRACEROUTE_ITEM_COST && isspace(*t)) {
 		// found a line starting with a hop number
 		latest_hop = static_cast<int>(hop);
