@@ -1,6 +1,6 @@
 /* 
  * Copyright 2004-2009 Rob Beverly
- * Copyright 2015-2022 The Regents of the University of California
+ * Copyright 2015-2022, 2026 The Regents of the University of California
  * All rights reserved.
  * 
  * This file is part of Spoofer.
@@ -32,7 +32,7 @@
 /****************************************************************************
    Author:      Rob Beverly <rbeverly at csail.mit.edu>
                 Ken Keys, CAIDA
-   Date:        $Date: 2022/03/04 23:48:10 $
+   Date:        2026-01-12 15:42:06 -0800
    Description: Spoofer - generate spoofed packets
 ****************************************************************************/
 
@@ -81,7 +81,6 @@ static bool multiThreaded = true;
 static bool multiThreaded = false;
 #endif
 
-static const char cvsid[] ATR_USED = "$Id: spoofer-prober.cc,v 1.241 2022/03/04 23:48:10 kkeys Exp $";
 
 static int sharePublic = -1; // initially unknown
 static int shareRemedy = -1; // initially unknown
@@ -456,6 +455,7 @@ void Sessions::printProgress() const {
     char *p = buf;
     p += sprintf(p, "%s", PROGRESS_PREFIX);
     for (unsigned i = 0; i < this->size(); i++) {
+	if (!(*this)[i]) continue;
 	SessionTracker *tracker = (*this)[i]->tracker;
 	if (!tracker || tracker->goal <= 0) continue;
 	p += sprintf(p, "  IPv%d: %d+%d/%d/%d", (*this)[i]->ipv,
@@ -1432,6 +1432,7 @@ void Session::generateFakeHello()
 		genSequence(seqno, SEQNOSIZE);
 		memcpy(seqno, "TEST", 4); // real seqno's never contain capitals
 		item->set_seqno(seqno, SEQNOSIZE);
+		item->set_hmac("@abcdefghijklmno", HMACSIZE);
 	    }
 	    work_est = resp.mutable_hello()->add_work_est();
 	    work_est->set_type(SPOOF);
@@ -1893,7 +1894,7 @@ static void banner() {
 #endif
     printf(">>\n");
 
-    for (start = PACKAGE_LONGDESC, len = 0; start[0]; start += len) {
+    for (start = PACKAGE_LONGDESC, len = 0; start[0]; start += len, len = 0) {
 	while (isspace(start[len])) ++start;
 	if ((len = strlen(start)) > 75) {
 	    len = 75;
@@ -1918,7 +1919,7 @@ NORETURN static void usage(char *prog, int exitcode, bool showHidden = false) {
     printf("-6          test IPv6 (default: both IPv4 and IPv6)\n");
     printf("-L<n>       spoof with layer <n> API: 3 (IP), 2 (link), or 0 (auto).\n");
 #ifdef HAVE_PTHREAD
-    printf("-1          run single-threaded (reduces network bandwidth and memory use\n");
+    printf("-1          run single-threaded (reduces network bandwidth and memory use)\n");
 #endif
 #ifdef HAVE_LIBSSL
     printf("--no-tls           connect to server without SSL/TLS\n");
@@ -2324,6 +2325,12 @@ int main(int argc, char **argv) {
 	    exitstatus = SP_EXIT_OK;
     }
 
+    // just to keep leak-check happy
+    for (unsigned i = 0; i < sessions.size(); i++) {
+        delete sessions[i];
+        sessions[i] = nullptr;
+    }
+
     displayResults(sessionkey.c_str());
 
     if (needUpgrade)
