@@ -1,5 +1,5 @@
 /* 
- * Copyright 2016-2022 The Regents of the University of California
+ * Copyright 2016-2022, 2026 The Regents of the University of California
  * All rights reserved.
  * 
  * This file is part of Spoofer.
@@ -21,7 +21,7 @@
 /****************************************************************************
    Author:      Rob Beverly <rbeverly at csail.mit.edu>
                 Ken Keys, CAIDA
-   Date:        $Date: 2022/02/07 19:48:23 $
+   Date:        2026-01-12 15:42:06 -0800
    Description: Traceroute wrapper
 ****************************************************************************/
 #ifndef _WIN32
@@ -33,7 +33,6 @@
 #include "prober.h"
 #include "routetracer.h"
 
-static const char cvsid[] ATR_USED = "$Id: unix_routetracer.cc,v 1.25 2022/02/07 19:48:23 kkeys Exp $";
 
 class UnixRouteTracer : public RouteTracer {
     public:
@@ -84,12 +83,6 @@ bool RouteTracer::init(IPv ipv)
     // Fall back to platform's traceroute
     std::vector<const char *> tr_candidate;
 
-    tr_candidate.push_back("/usr/local/sbin/traceroute");
-    tr_candidate.push_back("/usr/sbin/traceroute");
-    tr_candidate.push_back("/sbin/traceroute");
-    tr_candidate.push_back("/usr/bin/traceroute");
-    tr_candidate.push_back("/bin/traceroute");
-    tr_candidate.push_back("/usr/sbin/tracepath");
     if (ipv == IPv6) {
 	tr_candidate.push_back("/usr/local/sbin/traceroute6");
 	tr_candidate.push_back("/usr/sbin/traceroute6");
@@ -97,6 +90,15 @@ bool RouteTracer::init(IPv ipv)
 	tr_candidate.push_back("/usr/bin/traceroute6");
 	tr_candidate.push_back("/usr/sbin/tracepath6");
     }
+    // Note: native macOS /usr/sbin/tracroute interprets "::1" as an ipv4
+    // address and probes it (slowly), giving a false positive, so it is
+    // essential to try "traceroute6" first.
+    tr_candidate.push_back("/usr/local/sbin/traceroute");
+    tr_candidate.push_back("/usr/sbin/traceroute");
+    tr_candidate.push_back("/sbin/traceroute");
+    tr_candidate.push_back("/usr/bin/traceroute");
+    tr_candidate.push_back("/bin/traceroute");
+    tr_candidate.push_back("/usr/sbin/tracepath");
 
     static std::vector<const char *> options = {
 	"-N1", // send only 1 probe at a time (Linux)
